==> home-made-io-fastmag-sync-m2-0.21.1-3b0020.zip: Process/Entity/ToMagento/Product.php <==
<?php
/*******************************************************************************
 * Created by HOMEMADE.IO SAS.
 * Fastmag Sync  -  Connect Fastmag with your Magento
 *
 * Copyright (C) HOMEMADE.IO SAS, Inc - All Rights Reserved
 *
 * @author    Simon Laubet-Xavier <simon.laubetxavier@home-made.io>
 * @copyright 2020-2021 HOMEMADE.IO SAS
 * @date      2021-08-31
 ******************************************************************************/

namespace Fastmag\Sync\Process\Entity\ToMagento;

use DateTime;
use Exception;
use Fastmag\Sync\Process\Entity\ToMagento\Product\Variation as VariationEntity;
use Fastmag\Sync\Process\EntityInterface;
use IntlDateFormatter;
use Magento\Framework\DataObject;

/**
 * Class Product
 *
 * Entity class for product.
 * Save and transform data from Fastmag for Magento easy integration,
 * and transfer it from hydration worker to integration worker
 *
 * @method array|null getColorsI18n()
 * @method array|null getSizesI18n()
 * @method string|null getRef()
 * @method int|null getMagentoId()
 * @method string|null getStocksList()
 * @method string|null getVatFamily()
 * @method string|null getBrand()
 * @method string|null getSupplier()
 * @method string|null getFamily()
 * @method string|null getSubfamily()
 * @method string|null getSection()
 * @method string|null getSeason()
 * @method string|null getModel()
 * @method string|null getMaterial()
 * @method string|null getTheme()
 * @method float|null getWeight()
 * @method string|null getDesignation()
 * @method string|null getDesignationBis()
 * @method string|null getDescription()
 * @method string|null getWashing()
 * @method string|null getChlorine()
 * @method string|null getIroning()
 * @method string|null getDryCleaning()
 * @method string|null getDrying()
 * @method string|null getWater()
 * @method VariationEntity[]|null getVariations()
 * @method Product setRef(string $ref)
 * @method Product setColorsI18n(array $colorsI18n)
 * @method Product setSizesI18n(array $sizesI18n)
 * @method Product setMagentoId(int $magentoId)
 */
class Product extends DataObject implements EntityInterface
{
    /** @var string */
    public const EMPTY_COLOR_SIZE = '__empty__';

    /** @var IntlDateFormatter $dateFormatter */
    protected $dateFormatter;

    /**
     * Product constructor.
     *
     * @param array  $data
     */
    public function __construct(array $data = [])
    {
        parent::__construct($data);

        $this->dateFormatter = new IntlDateFormatter(
            'fr_FR',
            IntlDateFormatter::SHORT,
            IntlDateFormatter::NONE,
            'Europe/Paris',
            IntlDateFormatter::GREGORIAN,
            'dd/MM/yyyy'
        );
    }

    /**
     * Get visible web
     *
     * @return bool|null
     */
    public function getVisibleWeb()
    {
        return $this->getData('visible_web') !== null ? (bool)$this->getData('visible_web') : null;
    }

    /**
     * Get active
     *
     * @return bool|null
     */
    public function getActive()
    {
        return $this->getData('active') !== null ? (bool)$this->getData('active') : null;
    }

    /**
     * Get creation date
     *
     * @return string|null
     */
    public function getCreationDate()
    {
        $result = null;

        $creationDate = $this->getData('creation_date');

        if (!empty($creationDate) && !($creationDate instanceof DateTime)) {
            try {
                $this->setData('creation_date', new DateTime($creationDate));

                $result = $this->dateFormatter->format($this->getData('creation_date'));
            } catch (Exception $exception) {
                // Do nothing
            }
        }

        return $result;
    }

    /**
     * Get stock level
     *
     * @return int|null
     */
    public function getStockLevel()
    {
        return $this->getData('stock_level') !== null ? (int)$this->getData('stock_level') : null;
    }

    /**
     * Get vat rate
     *
     * @return float|null
     */
    public function getVatRate()
    {
        return $this->getData('vat_rate') !== null ? (float)$this->getData('vat_rate') : null;
    }

    /**
     * Get shop vat rate
     *
     * @return float|null
     */
    public function getShopVatRate()
    {
        return $this->getData('shop_vat_rate') !== null ? (float)$this->getData('shop_vat_rate') : null;
    }

    /**
     * Get i18ns
     *
     * @return Product[]
     */
    public function getI18ns()
    {
        return $this->getData('i18ns') !== [] ? $this->getData('i18ns') : null;
    }

    /**
     * Get i18n
     *
     * @param int $storeId
     *
     * @return Product|null
     */
    public function getI18n($storeId)
    {
        $i18ns = $this->getData('i18ns');

        return $i18ns[$storeId] ?? null;
    }

    /**
     * Returns i18n of a specific code for a specific store
     *
     * @param string $code
     * @param int    $storeId
     *
     * @return mixed
     */
    public function getDataI18n($code, $storeId)
    {
        $result = null;

        $i18n = $this->getI18n($storeId);

        if ($i18n !== null) {
            $result = $i18n->getData($code);
        }

        return $result;
    }

    /**
     * Returns all i18ns for a specific code
     *
     * @param string $code
     *
     * @return array
     */
    public function getDataI18ns($code)
    {
        $result = [];

        $i18ns = $this->getData('i18ns');

        if (is_array($i18ns) && count($i18ns) > 0) {
            /** @var Product $i18n */
            foreach ($i18ns as $storeId => $i18n) {
                if ($i18n->getData($code) !== null) {
                    $result[$storeId] = $i18n->getData($code);
                }
            }
        }

        return $result;
    }

    /**
     * Get variation
     *
     * @param string $color
     * @param string $size
     *
     * @return VariationEntity
     */
    public function getVariation($color, $size)
    {
        $result = null;

        $variations = $this->getData('variations');

        if (is_array($variations)
            && array_key_exists($color, $variations)
            && array_key_exists($size, $variations[$color])
        ) {
            $result = $variations[$color][$size];
        }

        return $result;
    }

    /**
     * Get variations as a flat array
     *
     * @return VariationEntity[]
     */
    public function getVariationsFlat()
    {
        $result = [];

        $variations = $this->getVariations();

        if (is_array($variations) && count($variations) > 0) {
            foreach ($variations as $sizeVariations) {
                foreach ($sizeVariations as $variation) {
                    $result[] = $variation;
                }
            }
        }

        return $result;
    }

    /**
     * Set VAT rate
     *
     * @param string $vatRate
     *
     * @return Product
     */
    public function setVatRate($vatRate)
    {
        return $this->setData('vat_rate', (float)$vatRate);
    }

    /**
     * Add i18n
     *
     * @param Product $i18n
     * @param string  $storeId
     *
     * @return Product
     */
    public function addI18n($i18n, $storeId)
    {
        $i18ns = $this->getData('i18ns');

        if ($i18ns === null) {
            $i18ns = [];
        }

        $i18ns[(int)$storeId] = $i18n;

        return $this->setData('i18ns', $i18ns);
    }

    /**
     * Check if a specific data is different from the standard value for a specific store.
     * If the data does not exist for a specific store, we consider that it's the same value as the standard one
     *
     * @param string $code
     * @param int    $storeId
     *
     * @return bool
     */
    public function isI18nDifferent($code, $storeId)
    {
        $result = false;

        $i18n = $this->getI18n($storeId);

        if ($i18n !== null) {
            $storeData = $this->getDataI18n($code, $storeId);

            if ($storeData !== null && $storeData !== $this->getData($code)) {
                $result = true;
            }
        }

        return $result;
    }

    /**
     * Add variation
     *
     * @param VariationEntity $variation
     * @param string          $color
     * @param string          $size
     *
     * @return Product
     */
    public function addVariation($variation, $color, $size)
    {
        if (empty($color)) {
            $color = self::EMPTY_COLOR_SIZE;
        }
        if (empty($size)) {
            $size = self::EMPTY_COLOR_SIZE;
        }

        $variations = $this->getData('variations');

        if ($variations === null || !array_key_exists($color, $variations)) {
            $variations[$color] = [$size => $variation];
        } else {
            $variations[$color][$size] = $variation;
        }

        return $this->setData('variations', $variations);
    }

    /**
     * @inheritDoc
     */
    public function export()
    {
        $result = [
            'colors_i18n'     => $this->getColorsI18n(),
            'sizes_i18n'      => $this->getSizesI18n(),
            'ref'             => $this->getRef(),
            'magento_id'      => $this->getMagentoId(),
            'visible_web'     => $this->getVisibleWeb(),
            'active'          => $this->getActive(),
            'creation_date'   => $this->getCreationDate(),
            'vat_rate'        => $this->getVatRate(),
            'stock_level'     => $this->getStockLevel(),
            'stocks_list'     => $this->getStocksList(),
            'brand'           => $this->getBrand(),
            'supplier'        => $this->getSupplier(),
            'family'          => $this->getFamily(),
            'subfamily'       => $this->getSubfamily(),
            'section'         => $this->getSection(),
            'season'          => $this->getSeason(),
            'model'           => $this->getModel(),
            'material'        => $this->getMaterial(),
            'theme'           => $this->getTheme(),
            'weight'          => $this->getWeight(),
            'designation'     => $this->getDesignation(),
            'designation_bis' => $this->getDesignationBis(),
            'description'     => $this->getDescription(),
            'washing'         => $this->getWashing(),
            'chlorine'        => $this->getChlorine(),
            'ironing'         => $this->getIroning(),
            'dry_cleaning'    => $this->getDryCleaning(),
            'drying'          => $this->getDrying(),
            'water'           => $this->getWater(),
            'i18ns'           => [],
            'variations'      => []
        ];

        $i18ns = $this->getData('i18ns');
        if (is_array($i18ns) && count($i18ns) > 0) {
            foreach ($i18ns as $storeId => $i18n) {
                $result['i18ns'][$storeId] = $i18n->export();
            }
        }

        $variations = $this->getVariations();
        if (is_array($variations) && count($variations) > 0) {
            foreach ($variations as $color => $variationsSizes) {
                if (!array_key_exists($color, $result['variations'])) {
                    $result['variations'][$color] = [];
                }

                /** @var VariationEntity $variation */
                foreach ($variationsSizes as $size => $variation) {
                    $result['variations'][$color][$size] = $variation->export();
                }
            }
        }

        return array_filter(
            $result,
            static function ($value) {
                return $value !== null && $value !== [];
            }
        );
    }
}

==> home-made-io-fastmag-sync-m2-0.25.0-3b5902.zip: Process/Entity/ToMagento/Product.php <==
<?php
/*******************************************************************************
 * Created by HOMEMADE.IO SAS.
 * Fastmag Sync  -  Connect Fastmag with your Magento
 *
 * Copyright (C) HOMEMADE.IO SAS, Inc - All Rights Reserved
 *
 * @author    Simon Laubet-Xavier <simon.laubetxavier@home-made.io>
 * @copyright 2020-2021 HOMEMADE.IO SAS
 * @date      2021-09-10
 ******************************************************************************/

namespace Fastmag\Sync\Process\Entity\ToMagento;

use DateTime;
use Exception;
use Fastmag\Sync\Process\Entity\ToMagento\Product\Variation as VariationEntity;
use Fastmag\Sync\Process\EntityInterface;
use IntlDateFormatter;
use Magento\Framework\DataObject;

/**
 * Class Product
 *
 * Entity class for product.
 * Save and transform data from Fastmag for Magento easy integration,
 * and transfer it from hydration worker to integration worker
 *
 * @method array|null getColorsI18n()
 * @method array|null getSizesI18n()
 * @method string|null getRef()
 * @method int|null getMagentoId()
 * @method string|null getStocksList()
 * @method string|null getVatFamily()
 * @method string|null getBrand()
 * @method string|null getSupplier()
 * @method string|null getFamily()
 * @method string|null getSubfamily()
 * @method string|null getSection()
 * @method string|null getSeason()
 * @method string|null getModel()
 * @method string|null getMaterial()
 * @method string|null getTheme()
 * @method float|null getWeight()
 * @method string|null getDesignation()
 * @method string|null getDesignationBis()
 * @method string|null getDescription()
 * @method string|null getWashing()
 * @method string|null getChlorine()
 * @method string|null getIroning()
 * @method string|null getDryCleaning()
 * @method string|null getDrying()
 * @method string|null getWater()
 * @method VariationEntity[]|null getVariations()
 * @method Product setRef(string $ref)
 * @method Product setColorsI18n(array $colorsI18n)
 * @method Product setSizesI18n(array $sizesI18n)
 * @method Product setMagentoId(int $magentoId)
 */
class Product extends DataObject implements EntityInterface
{
    /** @var string */
    public const EMPTY_COLOR_SIZE = '__empty__';

    /** @var IntlDateFormatter $dateFormatter */
    protected $dateFormatter;

    /**
     * Product constructor.
     *
     * @param array $data
     */
    public function __construct(array $data = [])
    {
        parent::__construct($data);

        $this->dateFormatter = new IntlDateFormatter(
            'fr_FR',
            IntlDateFormatter::SHORT,
            IntlDateFormatter::NONE,
            'Europe/Paris',
            IntlDateFormatter::GREGORIAN,
            'dd/MM/yyyy'
        );
    }

    /**
     * Get visible web
     *
     * @return bool|null
     */
    public function getVisibleWeb()
    {
        return $this->getData('visible_web') !== null ? (bool)$this->getData('visible_web') : null;
    }

    /**
     * Get active
     *
     * @return bool|null
     */
    public function getActive()
    {
        return $this->getData('active') !== null ? (bool)$this->getData('active') : null;
    }

    /**
     * Get creation date
     *
     * @return string|null
     */
    public function getCreationDate()
    {
        $result = null;

        $creationDate = $this->getData('creation_date');

        if (!empty($creationDate) && !($creationDate instanceof DateTime)) {
            try {
                $this->setData('creation_date', new DateTime($creationDate));

                $result = $this->dateFormatter->format($this->getData('creation_date'));
            } catch (Exception $exception) {
                // Do nothing
            }
        }

        return $result;
    }

    /**
     * Get stock level
     *
     * @return int|null
     */
    public function getStockLevel()
    {
        return $this->getData('stock_level') !== null ? (int)$this->getData('stock_level') : null;
    }

    /**
     * Get vat rate
     *
     * @return float|null
     */
    public function getVatRate()
    {
        return $this->getData('vat_rate') !== null ? (float)$this->getData('vat_rate') : null;
    }

    /**
     * Get shop vat rate
     *
     * @return float|null
     */
    public function getShopVatRate()
    {
        return $this->getData('shop_vat_rate') !== null ? (float)$this->getData('shop_vat_rate') : null;
    }

    /**
     * Get is new token
     *
     * @return bool|null
     */
    public function getIsNew()
    {
        return $this->getData('is_new') !== null ? (bool)$this->getData('is_new') : null;
    }

    /**
     * Get i18ns
     *
     * @return Product[]
     */
    public function getI18ns()
    {
        return $this->getData('i18ns') !== [] ? $this->getData('i18ns') : null;
    }

    /**
     * Get i18n
     *
     * @param int $storeId
     *
     * @return Product|null
     */
    public function getI18n($storeId)
    {
        $i18ns = $this->getData('i18ns');

        return $i18ns[$storeId] ?? null;
    }

    /**
     * Returns i18n of a specific code for a specific store
     *
     * @param string $code
     * @param int    $storeId
     *
     * @return mixed
     */
    public function getDataI18n($code, $storeId)
    {
        $result = null;

        $i18n = $this->getI18n($storeId);

        if ($i18n !== null) {
            $result = $i18n->getData($code);
        }

        return $result;
    }

    /**
     * Returns all i18ns for a specific code
     *
     * @param string $code
     *
     * @return array
     */
    public function getDataI18ns($code)
    {
        $result = [];

        $i18ns = $this->getData('i18ns');

        if (is_array($i18ns) && count($i18ns) > 0) {
            /** @var Product $i18n */
            foreach ($i18ns as $storeId => $i18n) {
                if ($i18n->getData($code) !== null) {
                    $result[$storeId] = $i18n->getData($code);
                }
            }
        }

        return $result;
    }

    /**
     * Get variation
     *
     * @param string $color
     * @param string $size
     *
     * @return VariationEntity
     */
    public function getVariation($color, $size)
    {
        $result = null;

        $variations = $this->getData('variations');

        if (is_array($variations)
            && array_key_exists($color, $variations)
            && array_key_exists($size, $variations[$color])
        ) {
            $result = $variations[$color][$size];
        }

        return $result;
    }

    /**
     * Get variations as a flat array
     *
     * @return VariationEntity[]
     */
    public function getVariationsFlat()
    {
        $result = [];

        $variations = $this->getVariations();

        if (is_array($variations) && count($variations) > 0) {
            foreach ($variations as $sizeVariations) {
                foreach ($sizeVariations as $variation) {
                    $result[] = $variation;
                }
            }
        }

        return $result;
    }

    /**
     * Set VAT rate
     *
     * @param string $vatRate
     *
     * @return Product
     */
    public function setVatRate($vatRate)
    {
        return $this->setData('vat_rate', (float)$vatRate);
    }

    /**
     * Add i18n
     *
     * @param Product $i18n
     * @param string  $storeId
     *
     * @return Product
     */
    public function addI18n($i18n, $storeId)
    {
        $i18ns = $this->getData('i18ns');

        if ($i18ns === null) {
            $i18ns = [];
        }

        $i18ns[(int)$storeId] = $i18n;

        return $this->setData('i18ns', $i18ns);
    }

    /**
     * Check if a specific data is different from the standard value for a specific store.
     * If the data does not exist for a specific store, we consider that it's the same value as the standard one
     *
     * @param string $code
     * @param int    $storeId
     *
     * @return bool
     */
    public function isI18nDifferent($code, $storeId)
    {
        $result = false;

        $i18n = $this->getI18n($storeId);

        if ($i18n !== null) {
            $storeData = $this->getDataI18n($code, $storeId);

            if ($storeData !== null && $storeData !== $this->getData($code)) {
                $result = true;
            }
        }

        return $result;
    }

    /**
     * Add variation
     *
     * @param VariationEntity $variation
     * @param string          $color
     * @param string          $size
     *
     * @return Product
     */
    public function addVariation($variation, $color, $size)
    {
        if (empty($color)) {
            $color = self::EMPTY_COLOR_SIZE;
        }
        if (empty($size)) {
            $size = self::EMPTY_COLOR_SIZE;
        }

        $variations = $this->getData('variations');

        if ($variations === null || !array_key_exists($color, $variations)) {
            $variations[$color] = [$size => $variation];
        } else {
            $variations[$color][$size] = $variation;
        }

        return $this->setData('variations', $variations);
    }

    /**
     * @inheritDoc
     */
    public function export()
    {
        $result = [
            'colors_i18n'     => $this->getColorsI18n(),
            'sizes_i18n'      => $this->getSizesI18n(),
            'ref'             => $this->getRef(),
            'magento_id'      => $this->getMagentoId(),
            'visible_web'     => $this->getVisibleWeb(),
            'active'          => $this->getActive(),
            'creation_date'   => $this->getCreationDate(),
            'vat_rate'        => $this->getVatRate(),
            'stock_level'     => $this->getStockLevel(),
            'stocks_list'     => $this->getStocksList(),
            'brand'           => $this->getBrand(),
            'supplier'        => $this->getSupplier(),
            'family'          => $this->getFamily(),
            'subfamily'       => $this->getSubfamily(),
            'section'         => $this->getSection(),
            'season'          => $this->getSeason(),
            'model'           => $this->getModel(),
            'material'        => $this->getMaterial(),
            'theme'           => $this->getTheme(),
            'weight'          => $this->getWeight(),
            'designation'     => $this->getDesignation(),
            'designation_bis' => $this->getDesignationBis(),
            'description'     => $this->getDescription(),
            'washing'         => $this->getWashing(),
            'chlorine'        => $this->getChlorine(),
            'ironing'         => $this->getIroning(),
            'dry_cleaning'    => $this->getDryCleaning(),
            'drying'          => $this->getDrying(),
            'water'           => $this->getWater(),
            'i18ns'           => [],
            'variations'      => []
        ];

        $i18ns = $this->getData('i18ns');
        if (is_array($i18ns) && count($i18ns) > 0) {
            foreach ($i18ns as $storeId => $i18n) {
                $result['i18ns'][$storeId] = $i18n->export();
            }
        }

        $variations = $this->getVariations();
        if (is_array($variations) && count($variations) > 0) {
            foreach ($variations as $color => $variationsSizes) {
                if (!array_key_exists($color, $result['variations'])) {
                    $result['variations'][$color] = [];
                }

                /** @var VariationEntity $variation */
                foreach ($variationsSizes as $size => $variation) {
                    $result['variations'][$color][$size] = $variation->export();
                }
            }
        }

        return array_filter(
            $result,
            static function ($value) {
                return $value !== null && $value !== [];
            }
        );
    }
}
